function DeepObjectTracker(objToTrack) {
    var $obj = objToTrack,
        $dirty = {},
        $objMap = {};
    
    this.getDirty = function() {
        return $dirty;
    }
    
    function isNonNative(obj) {
        return ("[object Object]" === Object.prototype.toString.apply(obj));
    }
    
    function proxyObj(obj) {
        
        function proxyProp(name) {
            var o = obj[name];
            if (isNonNative(o)) {
                proxyObj.call(o,o); //keep same scope for dirty
            }
            
            this.__defineGetter__(name, function(){ 
                var $d = $dirty[obj.uuid];
                
                return $d && $d.hasOwnProperty(name) ? $d[name] : $obj[name];
            });
            
            this.__defineSetter__(name, function(val){
                var v = obj[name];
                var $d = $dirty[obj.uuid];
                
                if ($d && $d.hasOwnProperty(name)) {
                    if (v === val) {
                        delete $d[name];
                    } else {
                        $d[name] = val;
                    }
                } else if (v !== val) {
                    if (!$d) $d = $dirty[obj.uuid] = {};
                    $d[name] = val;
                }
            });
        }
        
        function proxyFunc(name) {
            this[name] = function() {
                $obj[name].apply(this, arguments);
            }
        }
        
        function proxyArray(name) {
            
        }
        
        if (!obj.uuid) obj.uuid = uuid.v4();
        
        $objMap[obj.uuid] = obj;
        
        for (var mem in obj) {
            if (obj.hasOwnProperty(mem)) {
                if ('function' === typeof(obj[mem])) {
                    proxyFunc.call(this, mem);
                } else if (obj[mem] instanceof Array) {
                    proxyArray.call(this, mem);
                } else {
                    proxyProp.call(this, mem);
                }
            }
        }
    }
}

var uuid = require('node-uuid');

function SocketEmitter() {
    this.uuid = uuid.v4();
    
    this.push = function(sockets) {
        SocketEmitter.push(sockets, [this]);
    };
    
}

SocketEmitter.push = function(sockets, batch) {
    sockets.emit('object-push', batch);
}


exports = SocketEmitter;

