function engine(canvasContext)
{
  this.viewWindow = ({"x":0, "y":0, "width": 300, "height": 300});
  this.drawOn = canvasContext;
}

engine.prototype.Clear = function(){
  this.drawOn.fillStyle="#FFFFFF";
  this.drawOn.fillRect(0, 0, this.viewWindow.width, this.viewWindow.height);
};


engine.prototype.render = function(scene){
   var canvas = this.drawOn;
   var currentWindow = this.viewWindow;
   canvas.fillStyle="#FF0000";
   var layerList = scene.layers;
   
   for (l in layerList)
   {
       tileList = layerList[l].tiles;  
       spriteList = layerList[l].sprites;
       
       for (t in tileList)
       {
            (function(currentTile){
                if (withinBounds(currentWindow, currentTile))
                {
                    if (currentTile.src != null)
                    {
                      canvas.drawImage(images[currentTile.src], currentTile.x, currentTile.y);
                    }
                    else
                    {
                      canvas.beginPath();
                      canvas.arc(currentTile.x,currentTile.y,15,0,Math.PI*2,true);
                      canvas.closePath();
                      canvas.fill();
                    }
                }
            })(tileList[t]);
       }  
       
       for (s in spriteList)
       {
            (function(currentSprite){
                if (withinBounds(currentWindow, currentSprite))
                {
                    if (currentSprite.src != null)
                    {
                      var currentAnim = currentSprite.animations[currentSprite.curAnimation];
                      var targetFrame = currentAnim.frames[currentAnim.curFrame];
                      
                      canvas.drawImage(images[currentSprite.src],targetFrame.frameX, targetFrame.frameY, currentSprite.width, currentSprite.height, currentSprite.x, currentSprite.y,currentSprite.width, currentSprite.height); 
                    }
                }
            })(spriteList[s]);
       }
   }
};


function loadImages(sources, callback){
    
    var loadedImages = 0;
    var numImages = 0;
    // get num of sources
    for (var src in sources) {
        numImages++;
    }
    for (var src in sources) {
        images[src] = new Image();
        images[src].onload = function(){
            if (++loadedImages >= numImages) {
                callback(images);
            }
        };
        images[src].src = sources[src];
    }
}


function withinBounds(parentBox, childBox)
{
   var returnMe = false;
   
   if ((childBox.x >= parentBox.x) && (childBox.x <= parentBox.x + parentBox.width) && (childBox.y >= parentBox.y) && (childBox.y <= parentBox.y + parentBox.height))
   {
        returnMe = true;
   }
   
   return returnMe;
};